import numpy as np
import time
import mlflow
from .common import initialise_random_tensor

def train_kohonen_network_sam(input_data, n_max_iterations, width, height, feature_size,
                                 initial_learning_rate=0.1, use_mlflow=False):
    if use_mlflow:
        start_time = time.perf_counter()

    σ0 = max(width, height) / 2
    α0 = initial_learning_rate
    weights =  initialise_random_tensor(width, height, feature_size)
    λ = n_max_iterations / np.log(σ0)
    for t in range(n_max_iterations):
        σt = σ0 * np.exp(-t/λ)
        αt = α0 * np.exp(-t/λ)
        for vt in input_data:
            bmu = np.argmin(np.sum((weights - vt) ** 2, axis=2))
            bmu_x, bmu_y = np.unravel_index(bmu, (width, height))
            for x in range(width):
                for y in range(height):
                    di = np.sqrt(((x - bmu_x) ** 2) + ((y - bmu_y) ** 2))
                    θt = np.exp(-(di ** 2) / (2*(σt ** 2)))
                    weights[x, y] += αt * θt * (vt - weights[x, y])
    
    if use_mlflow:
        execution_time = time.perf_counter() - start_time
        mlflow.log_metric("execution_time", execution_time)
    return weights